///////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004 Intel Corporation 
// All rights reserved. 
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met: 
//
// * Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// * Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation 
// and/or other materials provided with the distribution. 
// * Neither name of Intel Corporation nor the names of its contributors 
// may be used to endorse or promote products derived from this software 
// without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
// OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <ipp.h>

#include "canny.h"
#include "Image.h"
#include "localbuffer.h"
#include "filter.h"

IppiSize Canny::GetKernelSize() const
{
	IppiSize kSize = {3,3};
	return kSize;
}
IppiPoint Canny::GetAnchor() const
{
	IppiPoint p = {1,1};
	return p;
}

int Canny::Go_(const Ipp8u* pSrc, int srcStep,
	Ipp8u* pDst, int dstStep,
	IppiSize dstROISize, int nChannels)
{
	if (nChannels != 1) return -1;

	// Allocate temporary images and buffers
	int bufSize;
	ippiCannyGetSize(dstROISize, &bufSize);
	int tmpSize = dstStep * dstROISize.height *
		sizeof(Ipp16s);
	buf_.SetMinAlloc(tmpSize*2 + bufSize);
	Ipp16s *pTmpX = (Ipp16s*)buf_.Alloc_8u(tmpSize);
	Ipp16s *pTmpY = (Ipp16s*)buf_.Alloc_8u(tmpSize);
	Ipp8u* pBuffer = buf_.Alloc_8u(bufSize);

	IppStatus st;
	st = ippiSobel3x3_Dx_8u16s_C1R(pSrc, srcStep,
		pTmpX, dstStep*sizeof(Ipp16s), dstROISize);
//	printf("%s\n",ippCoreGetStatusString(st));
	st = ippiSobel3x3_Dy_8u16s_C1R(pSrc, srcStep,
		pTmpY, dstStep*sizeof(Ipp16s), IPCV_ORIGIN_BL,
		dstROISize);
//	printf("%s\n",ippCoreGetStatusString(st));

	// Calculate Edges
	st = ippiCanny_16s8u_C1R(pTmpX, dstStep*2,
		pTmpY, dstStep*2, pDst, dstStep, dstROISize,
		lowThresh_, highThresh_, pBuffer);
//	printf("%s\n",ippCoreGetStatusString(st));

	buf_.ReleaseAll();
	return 0;
}